/*************************************************************
 *
 * tclogger.c:  Program to help me log my time card time for
 *              several different charge numbers.
 *
 * This program is distributed under the GNU General 
 * Public License (GPL).
 * See http://www.gnu.org/copyleft/gpl.html
 *
 * This software comes with NO warranty at all and I
 * cannot be held responsible for anything it may do.
 * I make no claims as to my ability to write programs.
 * You have the source code, so check it out.
 *
 * This program was developed on a Linux/GNU system using FREE
 * tools.  A special thanks to all programmers who developed
 * the following tools I used:
 *
 *   pilot-xfer-0.9.0
 *   gcc-2.7.2 cross compiler for Palm Connected Organizers
 *   gdb-4.16
 *   prctools-0.5.0
 *   pilot-template-1.31
 *   xcopilot-0.6.6
 *   pilrc-2.3a
 *
 * Kevin Dupree
 * kdupree@flash.net
 *************************************************************/

#pragma pack(2)

#include <Common.h>
#include <System/SysAll.h>
#include <UI/UIAll.h>

#include "callback.h"
#include "tclogger.h"
#include "util.h"

/*************************************************************
 *      Defines                                              *
 *************************************************************/
#define tcloggerCreator   'TCLg'
/* Define the minimum OS version allowed */
#define Version20    sysMakeROMVersion (2, 0, 0, 0, 0)
#define Version30    sysMakeROMVersion (3, 0, 0, 0, 0)
#define NUMTIMERS 7

/*************************************************************
 *	Type Definitions                                     *
 *************************************************************/

typedef struct {
  Boolean bRunning : 1;
  Long    lStartTime;
  Long    lAccumTime;
  Char    sDescription[ 21 ];
} TimerType;

typedef enum
{
  HOURS,
  HOURS_MIN,
  SECONDS
} TimerFormatType;

/* preferences state file. */
typedef struct {
  TimerFormatType AccumFormat;
  Char sInTime[ 41 ];
  Char sOutTime[ 41 ];
  TimerType TimerState[ NUMTIMERS ];
  Boolean bAlarmSet;
  Long    lAlarmTime;
  Boolean bDynUpdate;
  Boolean bOneTimerRun;
} TCLoggerPrefType;

/*************************************************************
 *	Global Variables                                     *
 *************************************************************/

static TimerType TimerData[ NUMTIMERS ];
static VoidHand hndInDesc;
static VoidHand hndOutDesc;
static VoidHand hndTime[ NUMTIMERS ];
static VoidHand hndDesc[ NUMTIMERS ];
static TimerFormatType AccumFormat = HOURS;
static Char sInTime[ 41 ];
static Char sOutTime[ 41 ];
SystemPreferencesType sysPrefs;   /* User's Pilot preferences. */
static Boolean bOKBold;
static VoidHand hndTotal;
static VoidHand hndAlarm;
static Boolean bAlarm;
static Long lAlarmTime;
static Boolean bDynUpdate;
static Boolean bCanUpdate = true;
static Boolean bOneTimerRun = true;


/*************************************************************
 *	Routines                                             *
 *************************************************************/
static Word StartApplication( void );
static void EventLoop( void );
static Boolean ApplicationHandleEvent( EventPtr );
static Boolean MainHandleEvent( EventPtr );
static Boolean MenuApplicationEvent( EventPtr );
static void StopApplication( void );

static void UpdateAllTimers( void );
static void ToggleTimerRun( Int );
static void StartTimer( Int );
static void StopTimer( Int );
static void UpdateTimer( Int, Boolean );
static void SetTimeFldEditable( Int, Boolean );
static void GetUsersTime ( Int );
static void FormatTime ( Long, char * );
static void FormatDate ( char * );
static Boolean TxtFieldHasFocus( Boolean );
static Word GetFocusID( void );
static void UpdateTotalTime( Boolean );
static Long ParseTime ( char * );
static void EditPrefs( void );
static void SetAlarm( void );
static void InitMainForm( void );
static void ShowAlarm( void );
static void CheckTimerEdits( void );


/*************************************************************
 *
 * NAME:        PilotMain
 *
 * DESCRIPTION: This is the main entry point for the
 *              application.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   --------   -----------
 *   kld    5 Nov 98   Initial Revision
 *   kld    6 Feb 99   Add setting of bold flag for font
 *                     of running timer.  For some reason the
 *                     bold font causes a "Fatal Exception" on
 *                     PalmOS 2.0 running on actual hardware but
 *                     not on PalmOS 2.0 debug ROM running on an
 *                     emulator, so disable it on PalmOS 2.0
 *   kld    17 Feb 99  Moved bold flag to StartApplication
 *   kld    23 Jun 99  Added Alarm Launch command processing
 *
 *************************************************************/
DWord PilotMain( Word cmd, Ptr cmdPBP, Word launchFlags )
{
  short err;
  Word error;

  CALLBACK_PROLOGUE

  err = CheckRomVersion( Version20, launchFlags );
  if (err) 
    {
      CALLBACK_EPILOGUE
      return (err);
    }

  switch (cmd)
    {
    case sysAppLaunchCmdNormalLaunch:
      error = StartApplication ();
      if (error) 
        {
          CALLBACK_EPILOGUE
          FrmCustomAlert( ErrorMsgAlert,
                          "Problem starting TCLogger!", NULL, NULL );
          return (error);
        }

      FrmGotoForm( tcloggerFormMain );

      EventLoop();
      StopApplication();
      break;

    case sysAppLaunchCmdAlarmTriggered:
      break;

    case sysAppLaunchCmdDisplayAlarm:
      ShowAlarm();
      break;

    default:
      break;
    }

  CALLBACK_EPILOGUE
  return 0;
}


/*************************************************************
 *
 * NAME:         StartApplication
 *
 * DESCRIPTION:  This routine loads the saved prefrences
 *               information and initializes global variables.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   --------   -----------
 *   kld    5 Nov 98   Initial Revision
 *   kld    17 Feb 99  Moved bold flag from PilotMain to here
 *   kld    10 May 99  Add setting of Alarm handle
 *   kld    10 May 99  Add alarm preferences
 *   kld    25 Jun 99  Updated to handle old preferences
 *
 *************************************************************/
static Word StartApplication( void )
{
  Word error = 1;
  TCLoggerPrefType  prefs;
  Int iPrefsVersion;
  Int prefsSize;
  Int i;
  DWord romVersion;

  /* Get the ROM version for which PalmOS version is being used. */
  FtrGet( sysFtrCreator, sysFtrNumROMVersion, &romVersion );
  bOKBold = (romVersion >= Version30);
    
  /* Get system preferences for In/Out date/time format */
  PrefGetPreferences( &sysPrefs );

  /* Read the preferences saved-state information. */
  prefsSize = sizeof (TCLoggerPrefType);
  iPrefsVersion = PrefGetAppPreferences (tcloggerCreator,
                                        TCLoggerPrefID, 
                                        &prefs,
                                        &prefsSize,
                                        true);

  if (iPrefsVersion == TCLoggerPrefsVersionNum)
    {
      AccumFormat = prefs.AccumFormat;
      StrCopy( sInTime, prefs.sInTime );
      StrCopy( sOutTime, prefs.sOutTime );
      for (i=0; i<NUMTIMERS; i++)
        TimerData[i] = prefs.TimerState[i];

      bAlarm = prefs.bAlarmSet;
      lAlarmTime = prefs.lAlarmTime;

      bDynUpdate = prefs.bDynUpdate;

      bOneTimerRun = prefs.bOneTimerRun;

      error = 0;
    }
  else if (iPrefsVersion == TCLoggerPrefsVersionNumOld)
    {
      FrmCustomAlert( InfoMsgAlert,
                      "Old preferences found, converting "\
                      "to new version.", NULL, NULL );

      /* Found an older version of preferences that
         did not have alarm or dynamic updates in it */
      AccumFormat = prefs.AccumFormat;
      StrCopy( sInTime, prefs.sInTime );
      StrCopy( sOutTime, prefs.sOutTime );
      for (i=0; i<NUMTIMERS; i++)
        TimerData[i] = prefs.TimerState[i];

      /* Set the default Alarm to off and Time to 8 Hours */
      bAlarm = false;
      lAlarmTime = 8L * 60L * 60L;

      bDynUpdate = false;
      bOneTimerRun = true;
      error = 0;
    }
  /* Set to defaults if no preferences found */
  else
  {
    FrmCustomAlert( InfoMsgAlert,
                    "No preferences found, using default values.",
                    NULL, NULL );
    AccumFormat = HOURS;
    StrCopy( sInTime, "None" );
    StrCopy( sOutTime, "None" );
    for (i=0; i<NUMTIMERS; i++)
      {
        TimerData[i].bRunning = false;
        TimerData[i].lStartTime = 0L;
        TimerData[i].lAccumTime = 0L;
        StrCopy( TimerData[i].sDescription,"" );
      }

    bAlarm = false;
    /* Set the default Alarm Time to 8 Hours */
    lAlarmTime = 8L * 60L * 60L;

    bDynUpdate = false;
    bOneTimerRun = true;

    error = 0;
  }

  hndInDesc = MemHandleNew( 41 );
  if (hndInDesc == NULL) return (1);
  hndOutDesc = MemHandleNew( 41 );
  if (hndOutDesc == NULL) return (1);

  for (i=0; i<NUMTIMERS; i++)
    {
      hndTime[i] = MemHandleNew( 7 );
      if (hndTime[i] == NULL) return (1);
      hndDesc[i] = MemHandleNew( 21 );
      if (hndDesc[i] == NULL) return (1);
    }

  hndTotal = MemHandleNew( 7 );
  if (hndTotal == NULL) return (1);

  return (error);
}


/*************************************************************
 *
 * NAME:        EventLoop
 *
 * DESCRIPTION: This routine is the event loop for processing
 *              events.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    29 Nov 98  Initial Revision
 *   kld    28 Jun 99  Added nilEvent request code
 *
 *************************************************************/
static void EventLoop( void )
{
  Word error;
  EventType event;
  Word wTicksPerSec = SysTicksPerSecond();
  Word wUpdate;

  do
    {
      /* Determine if Timers should be updated dynamically. */
      if ( bDynUpdate && bCanUpdate )
        wUpdate = 10 * wTicksPerSec; // Request nilEvent every 10 seconds
      else
        wUpdate = -1; // Do not want nilEvent
                      // NOTE: system requests nilEvent to flash cursor in text fields.

      // Get the next available event.
      EvtGetEvent( &event, wUpdate );

      // Give the system a chance to handle the event.
      if ( ! SysHandleEvent( &event ) )

        // Give the menu bar a chance to update and handle the event.
        if ( ! MenuHandleEvent( (void *) 0, &event, &error ) )

	  // Give the application a chance to handle the event.
          if ( ! ApplicationHandleEvent( &event ) )
	
            // Let the form object provide default handling of the event.
            FrmDispatchEvent( &event ); 
    }

  while (event.eType != appStopEvent);
}


/*************************************************************
 *
 * NAME:        EventLoop
 *
 * DESCRIPTION: This routine is the event loop for processing
 *              events.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    29 Nov 98  Initial Revision
 *   kld    6 Feb 99   Remove bold font for OS 2.0
 *   kld    17 Feb 99  Add call to update total time.
 *   kld    10 May 99  Add Prefs form
 *   kld    24 Jun 99  Removed Prefs form (moved to MenuApplicationEvent)
 *
 *************************************************************/
static Boolean ApplicationHandleEvent( EventPtr pEvent )
{
  FormPtr pFrm;
  Word	  idForm;
  Boolean bHandled = false;
  Char sID[ 10 ];

  if (pEvent->eType == frmLoadEvent)
    {
      /* Load the form resource specified in the event then activate the form. */
      idForm = pEvent->data.frmLoad.formID;
      pFrm = FrmInitForm( idForm );

      FrmSetActiveForm( pFrm );

      /* Set the event handler for the form.  The handler of the currently 
         active form is called by FrmDispatchEvent each time it receives an event. */
      switch (idForm)
        {
        case tcloggerFormMain:

          InitMainForm();

          FrmSetEventHandler( pFrm, MainHandleEvent );
          break;

        default:
          StrIToA( sID, idForm );
          FrmCustomAlert( UnknownFormAlert, sID, NULL, NULL );
          break;
        }
      bHandled = true;
    }

  return bHandled;
}


/*************************************************************
 *
 * NAME:        MainHandleEvent
 *
 * DESCRIPTION: This routine processes events for the main
 *              form.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    29 Nov 98  Initial Revision
 *   kld    28 Jun 99  Added winEnterEvent to prevent timers
 *                     from being updated when menu selected.
 *
 *************************************************************/
static Boolean MainHandleEvent( EventPtr pEvent )
{
  Boolean bHandled = false;
  FormPtr pFrm;
  CharPtr sBuff;
  FieldPtr pFld;
  Int i;
  Word wFocused;

  CALLBACK_PROLOGUE

  switch (pEvent->eType)
    {
    case ctlSelectEvent:
      if (pEvent->data.ctlSelect.controlID == idUpdateB)
        {
          UpdateAllTimers();
          bHandled = true;
        }
      else if((pEvent->data.ctlSelect.controlID >= idRunCB) &&
              (pEvent->data.ctlSelect.controlID < (idRunCB+NUMTIMERS)))
        {
          ToggleTimerRun( pEvent->data.ctlSelect.controlID - idRunCB );
          bHandled = true;
        }
      else if (pEvent->data.ctlSelect.controlID == idInB)
        {
          FormatDate( sInTime );

          pFld = Id2Ptr( idInFld );
          sBuff = (CharPtr) MemHandleLock( hndInDesc );
          StrCopy( sBuff, sInTime );
          MemHandleUnlock( hndInDesc );
          FldSetTextHandle ( pFld, (Handle) hndInDesc );
          FldDrawField( pFld );
          bHandled = true;
        }
      else if (pEvent->data.ctlSelect.controlID == idOutB)
        {
          FormatDate( sOutTime );

          pFld = Id2Ptr( idOutFld );
          sBuff = (CharPtr) MemHandleLock( hndOutDesc );
          StrCopy( sBuff, sOutTime );
          MemHandleUnlock( hndOutDesc );
          FldSetTextHandle ( pFld, (Handle) hndOutDesc );
          FldDrawField( pFld );
          bHandled = true;
        }
	
    case frmOpenEvent:
      pFrm = FrmGetActiveForm();

      FrmDrawForm( pFrm );

      bHandled = true;
      break;

    case keyDownEvent:
      /* Hard button pushed that did not power on the pilot */
      if (ChrIsHardKey( pEvent->data.keyDown.chr ))
        {
          if (! (pEvent->data.keyDown.modifiers & poweredOnKeyMask))
            {
              /* Launch the ToDo list application.
                 Done so that TCLogger could be mapped to ToDo's hardware
                 button.  Pressing ToDo's  hardware button with TCLogger
                 running will launch ToDo Application */
              Err err;     // Why is this var needed? See <AppLaunchCmd.h>!
              AppLaunchWithCommand(sysFileCToDo, 
                                   sysAppLaunchCmdNormalLaunch, 
                                   NULL);
              bHandled = true;
            }
          else
            {
              UpdateAllTimers();
              bHandled = true;
            }
        }
      if ( ! ( pEvent->data.keyDown.chr == 0 ) )
        {
          wFocused = FrmGetFocus( FrmGetActiveForm() );
          if ( wFocused >= idTimeFld &&
               wFocused < ( idTimeFld + NUMTIMERS ) )
            {
              if ( ! ( ( pEvent->data.keyDown.chr >= '0' &&
                         pEvent->data.keyDown.chr <= '9') ||
                       ( pEvent->data.keyDown.chr == '.') ||
                       ( pEvent->data.keyDown.chr == ':') ) )
                {
                  /* Character is not valid for a timer field,
                     so do not allow further processing */
                  bHandled = true;
                }
            }
        }
      break;

    case menuEvent:
      bHandled = MenuApplicationEvent( pEvent );
      break;

    case winEnterEvent :
      /* Make sure we do not update the fields that may be within the menu window.
         if this is not done, the times under the menu will be drawn on the menu. */
      bCanUpdate = ( pEvent->data.winExit.exitWindow != (WinHandle) FrmGetFormPtr( tcloggerFormMain ) );
      bHandled = true;
      break;

    case nilEvent:
      /* Update running timers and total time */
      for (i=0; i<NUMTIMERS; i++)
          if ( TimerData[i].bRunning )
            UpdateTimer( i, true );
      UpdateTotalTime( true );
      bHandled = true;
      break;

    default:
      bHandled = false;
      break;
    }

  CALLBACK_EPILOGUE

  return bHandled;
}


/*************************************************************
 *
 * NAME:        MenuApplicationEvent
 *
 * DESCRIPTION: This routine processes menu events for the
 *              main form.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    4 Dec 98   Initial Revision
 *   kld    8 Feb 99   Add check for edited time before export.
 *   kld    15 Feb 99  Added total accumulated time to
 *                     exported data.
 *   kld    17 Feb 99  Add call to update total time.
 *   kld    24 Jun 99  Added preference menu option.
 *   kld    28 Jun 99  Added day of week to export data.
 *   kld    28 Jun 99  Moved handle for export from global to here.
 *   kld     3 Aug 99  Added call to CheckTimerEdits
 *
 *************************************************************/
static Boolean MenuApplicationEvent( EventPtr pEvent )
{
  Boolean bHandled = false;
  CharPtr sFld;
  CharPtr sBuff;
  Char sTime[10];
  Char sDate[20];
  DateTimeType dtNow;
  FieldPtr pFld;
  Int i;
  Word wFocused;
  Int iTimer = 1;
  Long lTotalTime = 0L;
  Long lRound = 1L;
  VoidHand hndExport;
  CharPtr sWeekDay[7] = {"Sun", "Mon", "Tue", "Wed", 
                         "Thu", "Fri", "Sat"};

  if (pEvent->data.ctlSelect.controlID == idExportMenu)
    {
      hndExport = MemHandleNew( 400 );
      if (hndExport == NULL) {
        FrmCustomAlert( ErrorMsgAlert,
                        "Not enough memory to export data!", NULL, NULL );
        return (1);
      }

      sFld = (CharPtr) MemHandleLock( hndExport );
      StrCopy( sFld, "TCLogger:\t" );
      TimSecondsToDateTime( TimGetSeconds(), &dtNow );
      StrCat( sFld, sWeekDay[ DayOfWeek( dtNow.month, dtNow.day, dtNow.year ) ] );
      StrCat( sFld, " " );
      DateToAscii( dtNow.month, dtNow.day, dtNow.year, sysPrefs.longDateFormat, sDate );
      StrCat( sFld, sDate );
      StrCat( sFld, "\n\n" );

      /* Export Clock In Time */
      sBuff = (CharPtr) MemHandleLock( hndInDesc );
      StrCat( sFld, "In Time:\t\t" );
      StrCat( sFld, sBuff );
      StrCat( sFld, "\n\n" );
      MemHandleUnlock( hndInDesc );

      /* Determine rounding factor */
      if (AccumFormat == HOURS)
        lRound = 360L;
      else if (AccumFormat == HOURS_MIN)
        lRound = 60L;

      /* Export All the non-zero timers and descriptions */
      for (i=0; i<NUMTIMERS; i++)
        {
          if (FldDirty ( Id2Ptr( idTimeFld+i) ))
            GetUsersTime( i );          
          if (! TimerData[i].bRunning && 
              ( TimerData[i].lAccumTime > 0L ) )
            {
              /* Add to total, rounding is done so that fractional
                 parts do not add up and cause total to appear
                 too large */
              lTotalTime = lTotalTime + TimerData[i].lAccumTime - 
                (TimerData[i].lAccumTime%lRound);
              FormatTime( TimerData[i].lAccumTime, sTime );
              StrCat( sFld, sTime );
              StrCat( sFld, "\t" );
              /* Get the current description */
              pFld = Id2Ptr( idDescFld+i );
              sBuff = MemHandleLock( hndDesc[i] );
              StrCopy( TimerData[i].sDescription, sBuff );
              MemHandleUnlock( hndDesc[i] );
              StrCat( sFld, TimerData[i].sDescription );
              StrCat( sFld, "\n" );
            }
        }

      /* Export the total Time Charged */
      FormatTime( lTotalTime, sTime );
      StrCat( sFld, sTime );
      StrCat( sFld, "\tTotal Charged\n" );

      /* Export Clock Out Time */
      sBuff = (CharPtr) MemHandleLock( hndOutDesc );
      StrCat( sFld, "\nOut Time:\t" );
      StrCat( sFld, sBuff );
      StrCat( sFld, "\n" );
      MemHandleUnlock( hndOutDesc );

      MemHandleUnlock( hndExport );
      ExportToMemoPad( hndExport );
      MemHandleFree( hndExport );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idPrefsMenu)
    {
      EditPrefs();
      UpdateAllTimers();
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idClearFocusMenu)
    {
      if ( TxtFieldHasFocus( false ) )
        {
          wFocused = FrmGetFocus( FrmGetActiveForm() );
          wFocused = FrmGetObjectId( FrmGetActiveForm(), wFocused );
          if ( wFocused >= idTimeFld &&
               wFocused < ( idTimeFld + NUMTIMERS ) )
            {
              iTimer = wFocused - idTimeFld;
            }
          else if ( wFocused >= idDescFld &&
                    wFocused < ( idDescFld + NUMTIMERS ) )
            {
              iTimer = wFocused - idDescFld;
            }

          TimerData[iTimer].lStartTime = 0L;
          TimerData[iTimer].lAccumTime = 0L;
          UpdateTimer( iTimer, true );
          CheckTimerEdits();
          UpdateTotalTime( true );
        }
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idClearAllMenu)
    {
      for (i=0; i<NUMTIMERS; i++)
        {
          if (! TimerData[i].bRunning )
            {
              TimerData[i].lStartTime = 0L;
              TimerData[i].lAccumTime = 0L;
              UpdateTimer( i, true );
            }
        }
      CheckTimerEdits();
      UpdateTotalTime( true );

      StrCopy( sInTime, "None" );
      pFld = Id2Ptr( idInFld );
      sBuff = MemHandleLock( hndInDesc );
      StrCopy( sBuff, sInTime );
      MemHandleUnlock( hndInDesc );
      FldSetTextHandle ( pFld, (Handle) hndInDesc );
      FldDrawField( pFld );

      StrCopy( sOutTime, "None" );
      pFld = Id2Ptr( idOutFld );
      sBuff = MemHandleLock( hndOutDesc );
      StrCopy( sBuff, sOutTime );
      MemHandleUnlock( hndOutDesc );
      FldSetTextHandle ( pFld, (Handle) hndOutDesc );
      FldDrawField( pFld );

      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idClearIOMenu)
    {
      StrCopy( sInTime, "None" );
      pFld = Id2Ptr( idInFld );
      sBuff = MemHandleLock( hndInDesc );
      StrCopy( sBuff, sInTime );
      MemHandleUnlock( hndInDesc );
      FldSetTextHandle ( pFld, (Handle) hndInDesc );
      FldDrawField( pFld );

      StrCopy( sOutTime, "None" );
      pFld = Id2Ptr( idOutFld );
      sBuff = MemHandleLock( hndOutDesc );
      StrCopy( sBuff, sOutTime );
      MemHandleUnlock( hndOutDesc );
      FldSetTextHandle ( pFld, (Handle) hndOutDesc );
      FldDrawField( pFld );

      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idHoursMenu)
    {
      AccumFormat = HOURS;
      UpdateAllTimers();
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idHrMinMenu)
    {
      AccumFormat = HOURS_MIN;
      UpdateAllTimers();
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idSecMenu)
    {
      AccumFormat = SECONDS;
      UpdateAllTimers();
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idHelpMenu)
    {
      FrmHelp( tcloggerHelp1 );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idAboutMenu)
    {
      FrmHelp( tcloggerAbout1 );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idKeyBrdMenu)
    {
      /* See Keyboard.h for details */
      SysKeyboardDialog( kbdDefault );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idGHelpMenu)
    {
      /* See GraffitiReference.h for details */
      SysGraffitiReferenceDialog( referenceDefault );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idUndoMenu)
    {
      if ( TxtFieldHasFocus( true ) )
        FldUndo( Id2Ptr( GetFocusID() ) );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idCutMenu)
    {
      if ( TxtFieldHasFocus( true ) )
        FldCut( Id2Ptr( GetFocusID() ) );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idCopyMenu)
    {
      if ( TxtFieldHasFocus( true ) )
        FldCopy( Id2Ptr( GetFocusID() ) );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idPasteMenu)
    {
      if ( TxtFieldHasFocus( true ) )
        FldPaste( Id2Ptr( GetFocusID() ) );
      bHandled = true;
    }
  else if (pEvent->data.ctlSelect.controlID == idSelectMenu)
    {
      if ( TxtFieldHasFocus( true ) )
        FldSetSelection(Id2Ptr( GetFocusID() ), 
                        0,
                        FldGetTextLength( Id2Ptr( GetFocusID() ) ));
      bHandled = true;
    }
  return bHandled;
}


/*************************************************************
 *
 * NAME:        StopApplication
 *
 * DESCRIPTION: This routine saves the current state
 *              of the application.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    5 Nov 98   Initial Revision
 *   kld    29 Nov 98  Change event loop processing
 *   kld    10 May 99  Add alarm preferences
 *
 *************************************************************/
static void StopApplication( void )
{
  TCLoggerPrefType  prefs;
  Int i;
  CharPtr sDesc;

  prefs.AccumFormat = AccumFormat;

  sDesc = (CharPtr) MemHandleLock( hndInDesc );
  StrCopy( prefs.sInTime, sDesc );
  MemHandleUnlock( hndInDesc );

  sDesc = (CharPtr) MemHandleLock( hndOutDesc );
  StrCopy( prefs.sOutTime, sDesc );
  MemHandleUnlock( hndOutDesc );

  for (i=0; i<NUMTIMERS; i++)
    {
      sDesc = MemHandleLock( hndDesc[i] );
      StrCopy( TimerData[i].sDescription, sDesc );
      MemHandleUnlock( hndDesc[i] );
      if (FldDirty ( Id2Ptr( idTimeFld+i ) ))
          GetUsersTime( i );

      prefs.TimerState[i] = TimerData[i];
    }

  prefs.bAlarmSet = bAlarm;
  prefs.lAlarmTime = lAlarmTime;
  prefs.bDynUpdate = bDynUpdate;
  prefs.bOneTimerRun = bOneTimerRun;

  PrefSetAppPreferences (tcloggerCreator,
                         TCLoggerPrefID,
                         TCLoggerPrefsVersionNum,
                         &prefs, 
                         sizeof (TCLoggerPrefType),
                         true);

  /* FrmCloseAllForms will send all opened forms a
     frmCloseEvent. */
  FrmCloseAllForms();
}


/*************************************************************
 *
 * NAME:        ToggleTimerRun
 *
 * DESCRIPTION: This routine saves the current state
 *              of the application.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    1 Dec 98   Initial Development
 *   kld    17 Feb 99  Add call to update total time.
 *   kld    23 Jun 99  Add call to set/clear Alarm.
 *   kld     3 Aug 99  Added call to CheckTimerEdits
 *
 *************************************************************/
static void ToggleTimerRun( Int iTimer )
{
  int i;
  ControlPtr pCtl;

  if (TimerData[iTimer].bRunning)
    StopTimer( iTimer );
  else
    {
      if (bOneTimerRun)
        for (i=0; i<NUMTIMERS; i++)
          if (TimerData[i].bRunning && (i != iTimer))
            {
              // Need to uncheck checkbox here
              pCtl = (ControlPtr) Id2Ptr( idRunCB+i );
              CtlSetValue (pCtl, false);
              StopTimer( i );
            }
      StartTimer( iTimer );
    }

  CheckTimerEdits();
  UpdateTotalTime( true );

  /* Determine if Alarm should be Set*/
  SetAlarm();
}


/*************************************************************
 *
 * NAME:        StartTimer
 *
 * DESCRIPTION: This routine starts the given timer.  If
 *              the user has changed the accumulated time, 
 *              then it is read in and used (if the format
 *              is valid).
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    1 Dec 98   Initial Development
 *   kld    6 Feb 99   Remove bold font for OS 2.0
 *
 *************************************************************/
static void StartTimer( Int iTimer )
{
  FieldPtr pFld = Id2Ptr( idDescFld+iTimer);

  if (FldDirty ( Id2Ptr( idTimeFld+iTimer) ))
    GetUsersTime( iTimer );
 
  TimerData[iTimer].lStartTime = TimGetSeconds();

  if (bOKBold)
    FldSetFont( pFld, boldFont );
  FldDrawField( pFld );
  UpdateTimer( iTimer, true );

  TimerData[iTimer].bRunning = true;
  SetTimeFldEditable( iTimer, false );
}


/*************************************************************
 *
 * NAME:        StopTimer
 *
 * DESCRIPTION: This routine stops the given timer from 
 *              running.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    1 Dec 98   Initial Development
 *
 *************************************************************/
static void StopTimer( Int iTimer )
{
  FieldPtr pFld = Id2Ptr( idDescFld+iTimer);

  FldSetFont( pFld, stdFont );
  FldDrawField( pFld );
  UpdateTimer( iTimer, true );

  TimerData[iTimer].bRunning = false;
  SetTimeFldEditable( iTimer, true );
}


/*************************************************************
 *
 * NAME:        UpdateTimer
 *
 * DESCRIPTION: This routine updates the displayed time.
 *              If the timer is running it just displays
 *              what the current accumulated time is without
 *              updating global timers.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    1 Dec 98   Initial Development
 *
 *************************************************************/
static void UpdateTimer( Int iTimer, Boolean bUpdateDisplay )
{
  Long lTime = 0L;
  FieldPtr pFld;
  CharPtr sFld;

  if ( TimerData[iTimer].bRunning )
    {
      lTime = TimGetSeconds();
      TimerData[iTimer].lAccumTime = TimerData[iTimer].lAccumTime + 
        (lTime - TimerData[iTimer].lStartTime);
      TimerData[iTimer].lStartTime = lTime;
    }
  else if (FldDirty ( Id2Ptr( idTimeFld+iTimer) ))
    GetUsersTime( iTimer );
  lTime = TimerData[iTimer].lAccumTime;

  pFld = Id2Ptr(idTimeFld+iTimer);
  sFld = (CharPtr) MemHandleLock( hndTime[iTimer] );

  FormatTime( lTime, sFld );

  MemHandleUnlock( hndTime[iTimer] );
  FldSetTextHandle( pFld, (Handle) hndTime[iTimer] );
  if ( bUpdateDisplay )
    FldDrawField( pFld );
}


/*************************************************************
 *
 * NAME:        UpdateAllTimers
 *
 * DESCRIPTION: This routine updates all timers accumulated
 *              time.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    1 Dec 98   Initial Development
 *   kld    17 Feb 99  Add call to update total time.
 *
 *************************************************************/
static void UpdateAllTimers( void )
{
  Int i;

  for (i=0; i<NUMTIMERS; i++)
    UpdateTimer( i, true );
  UpdateTotalTime( true );
}


/*************************************************************
 *
 * NAME:        SetTimeFldEditable
 *
 * DESCRIPTION: This routine will mark a time field as editable
 *              based on the passed in value.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    2 Dec 98   Initial development
 *
 *************************************************************/
static void SetTimeFldEditable( Int iTimer, Boolean bEdit )
{
    FieldPtr pTimeFld = Id2Ptr( idTimeFld + iTimer );
    FieldAttrType attr;

    FldGetAttributes( pTimeFld, &attr );
    attr.editable = bEdit;
    FldSetAttributes( pTimeFld, &attr );
    FldReleaseFocus( pTimeFld );
}


/*************************************************************
 *
 * NAME:        GetUsersTime
 *
 * DESCRIPTION: This routine will read in the user entered
 *              time in the format H:M, H.H or SEC.  If an error
 *              occurs the last saved accumulated time will be
 *              restored in the time field.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    2 Dec 98   Initial development
 *   kld    11 May 99  Moved parsing code to ParseTime
 *
 *************************************************************/
static void GetUsersTime ( Int iTimer )
{
  Long lTime = 0L;
  CharPtr pTime = FldGetTextPtr( Id2Ptr( idTimeFld + iTimer ) );
  Char sBuff[3];

  lTime = ParseTime( pTime );
  /* If there is not an error, the users time will be used
     if there is an error the old accumulated time will be
     used. */
  if (lTime < 0)
    {
      StrIToA( sBuff, iTimer+1 );
      FrmCustomAlert( TimeParseAlert, sBuff, NULL, NULL );
    }
  else
    {
      TimerData[ iTimer ].lAccumTime = lTime;
    }
}

/*************************************************************
 *
 * NAME:        ParseTime
 *
 * DESCRIPTION: This routine will parse a character string for
 *              time in the format H:M, H.H or SEC.  The return
 *              value will be the number of seconds represented
 *              by the character string.  A negative number
 *              indicates an error occured.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    11 May 99  Initial development
 *   kld    15 Jul 99  Correct so lTime is not updated when
 *                     still looping through text
 *
 *************************************************************/
static Long ParseTime ( Char *pTime )
{
  Boolean bError = false;
  Boolean bHoursOnly = false;
  Boolean bMinutes = false;
  Int iDecimal = 1;
  Long lCurrent = 0L;
  Long lMinutes = 0L;
  Long lTime = -1L;
  Int length = StrLen( pTime );
  Int i;

  if (pTime)
    {
      for (i=0; i<length; i++)
        {
          if ( *pTime >= '0' && *pTime <= '9' )
            {
              lCurrent = ( lCurrent * 10L ) + ( *pTime - '0' );
              if (bHoursOnly)
                iDecimal*=10;
            }
          else if (*pTime == '.' && (! bHoursOnly) && (! bMinutes))
            {
              bHoursOnly = true;
            }
          else if (*pTime == ':' && (! bHoursOnly) && (! bMinutes))
            {
              bMinutes = true;
              /* Convert hours to minutes */
              lMinutes = lCurrent * 60L;
              lCurrent = 0L;
            }
          else if (*pTime == ' ')
            /* Skip whitespace */
            ;
          else
            bError = true;
          pTime++;
        }
      /* If there is not an error, the users time will be used
         if there is an error the old accumulated time will be
         used. */
      if (!bError)
        {
          if ( bHoursOnly )
            /* Convert hours to seconds */
            lTime = ((lCurrent * 3600L) / (Long) (iDecimal));
          else if ( bMinutes )
            /* Convert minutes to seconds */
            lTime = (lCurrent + lMinutes) * 60L;
          else
            lTime = lCurrent;
        }
    }
  return lTime;
}

/*************************************************************
 *
 * NAME:        ICat
 *
 * DESCRIPTION: This routine Cat the given integer to the
 *              given string
 *
 *************************************************************/
void ICat( UInt ui, char *s )
{
  Char sBuff[ 10 ];

  StrIToA( sBuff, ui );
  StrCat( s, sBuff );
}

/*************************************************************
 *
 * NAME:        ZeroPadICat
 *
 * DESCRIPTION: This routine Cat the given integer to the
 *              given string and will pad the 10s place with
 *              0 if needed.
 *
 *************************************************************/
static void ZeroPadICat( UInt ui, char *s )
{
  if ( ui < 10 )
    StrCat( s, "0" );

  ICat( ui, s );
}

/*************************************************************
 *
 * NAME:        FormatTime
 *
 * DESCRIPTION: This routine will format the given time in
 *              the selected format of H.H, H:M, SEC
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    3 Dec 98   Initial development
 *
 *************************************************************/
static void FormatTime( Long lTime, char *sTime )
{
  UInt uiHours, uiMin, uiFrac;

  if (AccumFormat == HOURS)
    {
      uiHours = (UInt) (lTime / 3600L );
      lTime = (Long) ( lTime % 3600L );
      uiFrac = (UInt) (lTime / 360L );

      StrCopy ( sTime, "" );
      ICat( uiHours, sTime );
      StrCat( sTime, "." );
      ICat( uiFrac, sTime );
    }
  else if (AccumFormat == HOURS_MIN)
    {
      uiHours = (UInt) (lTime / 3600L );
      lTime = (Long) ( lTime % 3600L );
      uiMin = (UInt) (lTime / 60L );

      StrCopy ( sTime, "" );
      ICat( uiHours, sTime );
      StrCat( sTime, ":" );
      ZeroPadICat( uiMin, sTime );
    }
  else
    {
      StrIToA( sTime, (Int) lTime);
    }
}


/*************************************************************
 *
 * NAME:        FormatDate
 *
 * DESCRIPTION: This routine will format the given time for
 *              use with the clock in/out function.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    3 Dec 98   Initial development
 *
 *************************************************************/
static void FormatDate ( char *sDesc )
{
  CharPtr ptrEOS;
  DateTimeType dtNow;

  TimSecondsToDateTime( TimGetSeconds(), &dtNow );

  DateToAscii( dtNow.month, dtNow.day, dtNow.year, sysPrefs.dateFormat, sDesc );
  StrCat( sDesc, " " );
  ptrEOS = sDesc + StrLen( sDesc );
  TimeToAscii( dtNow.hour, dtNow.minute, sysPrefs.timeFormat, ptrEOS );
}


/*************************************************************
 *
 * NAME:        TxtFieldHasFocus
 *
 * DESCRIPTION: This routine will determine if a text field
 *              currently has focus.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    3 Dec 98   Initial development
 *
 *************************************************************/
static Boolean TxtFieldHasFocus( Boolean bCheckInOut )
{
  Word    wFocused  = FrmGetFocus( FrmGetActiveForm() );
  Boolean bIsTxtFld = 0;

  if ( wFocused != -1 )
    {
      wFocused  = FrmGetObjectId( FrmGetActiveForm(), wFocused );
      bIsTxtFld = ( ( wFocused >= idTimeFld &&
                      wFocused < ( idTimeFld + NUMTIMERS ) ) ||
                    ( wFocused >= idDescFld &&
                      wFocused < ( idDescFld + NUMTIMERS ) ) ||
                    ( bCheckInOut && ( wFocused == idInFld ||
                                       wFocused == idOutFld ) ) );
    }

  if ( !bIsTxtFld )
    {
      SndPlaySystemSound( sndError );
      FrmCustomAlert( ErrorMsgAlert, 
                      "Cannot perform request.  Please select a "\
                      "field and try again.", NULL, NULL );
    }

  return( bIsTxtFld );
}


/*************************************************************
 *
 * NAME:        GetFocusID
 *
 * DESCRIPTION: This routine gets the id of the frame object
 *              that currently had data entry focus.
 *
 *************************************************************/
static Word GetFocusID( void )
{
  Word wFocused = FrmGetFocus( FrmGetActiveForm() );

  if ( wFocused != -1 )
    wFocused = FrmGetObjectId( FrmGetActiveForm(), wFocused );
    
  return( wFocused );
}


/*************************************************************
 *
 * NAME:        UpdateTotalTime
 *
 * DESCRIPTION: This routine will update the total time
 *              displayed.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    17 Feb 99  Initial development
 *   kld     3 Aug 99  Added checking for timer focus so
 *                     that it is read in only if needed.
 *                     Currently only dynamic updating requires
 *                     this to be off so that the user has
 *                     time to complete their edits before
 *                     the data is read in.
 *
 *************************************************************/
static void UpdateTotalTime( Boolean bUpdateDisplay )
{
  CharPtr sFld;
  FieldPtr pFld;
  Long lTotalTime = 0L;
  Long lRound = 1L;
  Int i;

  if (AccumFormat == HOURS)
    lRound = 360L;
  else if (AccumFormat == HOURS_MIN)
    lRound = 60L;

  for (i=0; i<NUMTIMERS; i++)
    {
      if ( TimerData[i].lAccumTime > 0L )
        /* Add to total, rounding is done so that fractional
           parts do not add up and cause total to appear
           too large */
        lTotalTime = lTotalTime + TimerData[i].lAccumTime - 
          (TimerData[i].lAccumTime%lRound);
    }

  pFld = Id2Ptr( idTotalFld );
  sFld = (CharPtr) MemHandleLock( hndTotal );

  FormatTime( lTotalTime, sFld );

  MemHandleUnlock( hndTotal );
  FldSetTextHandle( pFld, (Handle) hndTotal );

  if ( bUpdateDisplay )
    FldDrawField( pFld );
}



/*************************************************************
 *
 * NAME:        EditPrefs
 *
 * DESCRIPTION: This routine will initialize and display the
 *              Preferences modal dialog box.  Upon exit of
 *              dialog box, it determine if any preferences
 *              should be updated.  It is also responsible
 *              for determining of the Alarm should be set
 *              or cleared.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    23 Jun 99  Initial development
 *   kld     3 Aug 99  Clear alarm time if time parse error
 *                     occurs.
 *
 *************************************************************/
static void EditPrefs( void )
{
  FormPtr previousForm = FrmGetActiveForm();
  FormPtr pFrm = FrmInitForm(idPrefsFrm);
  ControlPtr pHoursCtl = FrmGetObjectPtr( pFrm,
                                          FrmGetObjectIndex( pFrm, idHoursPB ) );
  ControlPtr pHrMinCtl = FrmGetObjectPtr( pFrm,
                                          FrmGetObjectIndex( pFrm, idHrMinPB ) );
  ControlPtr pDynUpCtl = FrmGetObjectPtr( pFrm,
                                          FrmGetObjectIndex( pFrm, idDynUpCB ) );
  ControlPtr pAlarmCtl = FrmGetObjectPtr( pFrm,
                                          FrmGetObjectIndex( pFrm, idAlarmCB ) );
  ControlPtr pOneRunCtl = FrmGetObjectPtr( pFrm,
                                          FrmGetObjectIndex( pFrm, idOneRunCB ) );
  FieldPtr pAlarmFld = FrmGetObjectPtr( pFrm,
                                        FrmGetObjectIndex( pFrm, idAlarmFld ) );
  CharPtr pAlarm;
  Word hitButton;
  CharPtr sTime;
  Long lTime;

  hndAlarm = MemHandleNew( 7 );
  if (hndAlarm == NULL) return;

  FrmSetActiveForm( pFrm );

  // Set an event handler, if you wish, with FrmSetEventHandler

  /* Initialize the current values */
  CtlSetValue( pHoursCtl, false );
  CtlSetValue( pHrMinCtl, false );
  switch (AccumFormat)
    {
    case HOURS:
      CtlSetValue( pHoursCtl, true );
      break;
    case HOURS_MIN:
      CtlSetValue( pHrMinCtl, true );
      break;
    case SECONDS:
      break;  // Can't change this one prefs screen
    }
  CtlSetValue( pOneRunCtl, bOneTimerRun );
  CtlSetValue( pDynUpCtl, bDynUpdate );
  CtlSetValue( pAlarmCtl, bAlarm );
  sTime = (CharPtr) MemHandleLock( hndAlarm );
  FormatTime( lAlarmTime, sTime );
  MemHandleUnlock( hndAlarm );
  FldSetTextHandle( pAlarmFld, (Handle) hndAlarm );

  hitButton = FrmDoDialog(pFrm);

  /* Get alarm checkbox and entered Alarm Time before
     Deleting the form. */
  if ( CtlGetValue( pHoursCtl ) )
    AccumFormat = HOURS;
  else
    AccumFormat = HOURS_MIN;
  bOneTimerRun = CtlGetValue( pOneRunCtl );
  bDynUpdate = CtlGetValue( pDynUpCtl );
  bAlarm = CtlGetValue( pAlarmCtl );
  pAlarm = FldGetTextPtr( Id2Ptr( idAlarmFld ) );
  lTime = ParseTime( pAlarm );

   if (previousForm)
    FrmSetActiveForm( previousForm );

  FrmDeleteForm( pFrm );
  if (hitButton == idOkB)
    {
      /* Save current preference Settings
         If there is not an error, the users time will be used
         if there is an error the old alarm time will be
         used. */
      if (lTime < 0)
        {
          lAlarmTime = 0L;
          FrmCustomAlert( TimeParseAlert, "Alarm", NULL, NULL );
        }
      else
        lAlarmTime = lTime;

      SetAlarm();
    }
}

/*************************************************************
 *
 * NAME:        SetAlarm
 *
 * DESCRIPTION: This routine will set an Alarm for the
 *              requested time.  The alarm is set based
 *              on the current time, and even accounts
 *              for more than 1 timer running.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    23 Jun 99  Initial development
 *
 *************************************************************/
static void SetAlarm( void )
{
  ULong lCurrentTime;
  ULong lTotalTime = 0L;
  UInt              uiCardNo;
  LocalID           dbID;
  DmSearchStateType dmsDummy;
  Int i;
  Int iRunCnt = 0;
  ULong lAlarm = 0L;
  Long lTime;
  Long lRound = 1L;

  /* Determine rounding factor */
  if (AccumFormat == HOURS)
    lRound = 360L;
  else if (AccumFormat == HOURS_MIN)
    lRound = 60L;

  lCurrentTime = TimGetSeconds();
  for (i=0; i<NUMTIMERS; i++)
    {
      if (TimerData[i].bRunning)
        {
          lTime = TimerData[i].lAccumTime + lCurrentTime - 
            TimerData[i].lStartTime;
          iRunCnt++;
        }
      else
        {
          /* Round time so that fractional parts do not cause alarm
             to happen too early  ONLY done for stopped timers */
          lTime = TimerData[i].lAccumTime - 
            TimerData[i].lAccumTime%lRound;
        }
      lTotalTime = lTotalTime + lTime;
    }
  /* Make sure a timer is running, alarms are activated and that
     the alarm time has not already passed */
  if ((iRunCnt > 0) && bAlarm && (lAlarmTime > lTotalTime))
    lAlarm = lCurrentTime + ((lAlarmTime - lTotalTime) / iRunCnt);
  else
    lAlarm = 0L; // Clear Alarm
  if ( !DmGetNextDatabaseByTypeCreator( true, 
                                        &dmsDummy, 
                                        0, 
                                        tcloggerCreator, 
                                        true, 
                                        &uiCardNo, 
                                        &dbID ) )
    {
      if ( AlmSetAlarm( uiCardNo, dbID, 0, lAlarm, 0 ) )
        FrmCustomAlert( ErrorMsgAlert, 
                        "Alarm could not be set.", NULL, NULL );
    }
}


/*************************************************************
 *
 * NAME:        InitMainForm
 *
 * DESCRIPTION: This routine will initialize all fields in
 *              the Main TCLogger form.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    24 Jun 99  Initial development 
 *                     (code moved from ApplicationHandleEvent )
 *
 *************************************************************/
static void InitMainForm( void )
{
  CharPtr sBuff;
  FieldPtr pFld;
  ControlPtr pCtl;
  Int i;

  /* Set the running checkbox */
  for (i=0; i<NUMTIMERS; i++)
    {
      /* Set the Run checkbox */
      pCtl = (ControlPtr) Id2Ptr( idRunCB+i );
      CtlSetValue (pCtl, TimerData[i].bRunning);
      SetTimeFldEditable( i, !TimerData[i].bRunning );

      /* Set the timers description */
      pFld = Id2Ptr( idDescFld+i );
      sBuff = MemHandleLock( hndDesc[i] );
      StrCopy( sBuff, TimerData[i].sDescription );
      MemHandleUnlock( hndDesc[i] );
      FldSetTextHandle( pFld, (Handle) hndDesc[i] );

      /* If running highlite description font and 
         update the time accumulated */
      if (TimerData[i].bRunning && bOKBold)
        FldSetFont( pFld, boldFont );
      UpdateTimer( i, false );
    }
  /* Set the total accumulated time */
  UpdateTotalTime( false );

  /* Set the clock in time */
  pFld = Id2Ptr( idInFld );
  sBuff = MemHandleLock( hndInDesc );
  StrCopy( sBuff, sInTime );
  MemHandleUnlock( hndInDesc );
  FldSetTextHandle ( pFld, (Handle) hndInDesc );

  /* Set the clock out time */

  pFld = Id2Ptr( idOutFld );
  sBuff = MemHandleLock( hndOutDesc );
  StrCopy( sBuff, sOutTime );
  MemHandleUnlock( hndOutDesc );
  FldSetTextHandle ( pFld, (Handle) hndOutDesc );
}


/*************************************************************
 *
 * NAME:        ShowAlarm
 *
 * DESCRIPTION: This routine displays alarm info when it goes off.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld    29 Jun 99  Initial development 
 *
 *************************************************************/
static void ShowAlarm( void )
{
  FormPtr pFrm;
  FormPtr previousForm = FrmGetActiveForm();

  SndPlaySystemSound( sndAlarm );

  pFrm = FrmInitForm(idAlarmFrm);
  FrmSetActiveForm( pFrm );
  FrmDoDialog(pFrm);
  FrmDeleteForm( pFrm );

  if (previousForm)
    FrmSetActiveForm( previousForm );
}


/*************************************************************
 *
 * NAME:        CheckTimerEdits
 *
 * DESCRIPTION: This routine will check for time that has
 *              been edited and update it.
 *
 * REVISION HISTORY:
 *   Name   Date       Description
 *   ----   ---------  -----------
 *   kld     4 Aug 99  Initial development.
 *
 *************************************************************/
static void CheckTimerEdits( void )
{
  Int i;
  Int iTimerFocused = -1;
  Word wFocused = FrmGetFocus( FrmGetActiveForm() );

  if ( wFocused != -1 )
    {
      wFocused = FrmGetObjectId( FrmGetActiveForm(), wFocused );
      if ( wFocused >= idTimeFld &&
           wFocused < ( idTimeFld + NUMTIMERS ) )
        iTimerFocused = wFocused - idTimeFld;
    }
  for (i=0; i<NUMTIMERS; i++)
    {
      /* Only read in new timer data if it has changed and does
         not have focus unless forced */
      if ( FldDirty ( Id2Ptr( idTimeFld+i) ) &&
           ( i != iTimerFocused) )
        GetUsersTime( i );
    }
}
